#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QFileDialog>
#include <QSettings>
#include <QDesktopServices>
#include <QFileInfo>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    connect(ui->startFileBrowse, SIGNAL(clicked()), this, SLOT(startFileBrowse()));
    connect(ui->runButton, SIGNAL(clicked()), this, SLOT(run()));
    connect(this, SIGNAL(newWarning(QString)), ui->textBrowser, SLOT(append(QString)));
    connect(this, SIGNAL(startRun()), ui->textBrowser, SLOT(clear()));

    FindEvents();
    for (std::map<QString,QString>::iterator iter = m_Events.begin(); iter != m_Events.end(); iter++)
        ui->FinishedCombo->addItem(iter->first);

    QSettings settings(QSettings::IniFormat,  QSettings::UserScope, "undy","SI Check");
    settings.beginGroup("General");
    ui->StartFileEdit->setText(settings.value("Start File","").toString());
    settings.endGroup();
}

MainWindow::~MainWindow()
{
    QSettings settings(QSettings::IniFormat,  QSettings::UserScope, "undy","SI Check");
    settings.beginGroup("General");
    settings.setValue("Start File",ui->StartFileEdit->text());
    settings.setValue("Finish File",ui->FinishedCombo->currentText());
    settings.endGroup();

    delete ui;
}

void MainWindow::FindEvents()
{
    // Need to find Or directory, then parse all the event subfolders and extract the event name and rawlog file name
    QString dir = QDesktopServices::storageLocation(QDesktopServices::HomeLocation);
    dir += "\\Or";

    QDir orDir(dir);
    QStringList dirs = orDir.entryList(QDir::Dirs);
    for (int i = 0; i < dirs.count(); i++)
        {
        QRegExp re("^[0-9]*$");
        if (re.exactMatch(dirs.at(i)))
            {
            QFileInfo compFileInfo(dir + "\\" + dirs.at(i) + "\\Competition.csv");
            QFileInfo logFileInfo(dir + "\\" + dirs.at(i) + "\\resultslog.csv");
            if (compFileInfo.exists() && logFileInfo.exists())
                {
                // Extract comp name from log-file
                QFile tfile2(compFileInfo.absoluteFilePath());
                if (tfile2.open(QIODevice::ReadOnly))
                    {
                    QByteArray bytes = tfile2.readAll();
                    QString temp(bytes);
                    QStringList parts = temp.split(";");
                    this->m_Events[dirs.at(i) + "-" + parts.at(1)] = logFileInfo.absoluteFilePath();
                    tfile2.close();
                    }
                else
                    {
                    QString warning = QString("Couldn't open %1, error %2").arg(compFileInfo.absolutePath()).arg(tfile2.errorString());
                    emit newWarning(warning);
                    }
                }
            }
        }

}

void MainWindow::startFileBrowse()
    {
    QString startFile = QFileDialog::getOpenFileName(this,
    tr("Open Start Times File"), m_Dir, tr("csv Files (*.csv *.txt)"));

    if (!startFile.isEmpty())
        ui->StartFileEdit->setText(startFile);
    }


bool MainWindow::MatchTime(QString timeFromUnit, QString timeFromCard)
{
    if (timeFromUnit == timeFromCard)
        return true;

    if (timeFromUnit.length() != 8 || (timeFromCard.length() != 8 && timeFromCard.length() !=7))
        {
        QString warning = QString("Garbage times supplied to compare %1,%2").arg(timeFromUnit).arg(timeFromCard);
        emit newWarning(warning);
        return false;
        }

    if (timeFromCard.right(5) == timeFromUnit.right(5))
        {
        QStringList unitTimes = timeFromUnit.split(":");
        QStringList cardTimes = timeFromCard.split(":");

        return (unitTimes.at(0).toInt() == cardTimes.at(0).toInt() + 12);
        }
    return false;
}

void MainWindow::LoadStarts(QString& a_Starts)
{
    QStringList records = a_Starts.split(QRegExp("\n|\r|\r\n"));
    m_IgnoredStarts = 0;

    // Rely on the fact that they are stored in chronological order
    for (int i = 1; i < records.count(); i++)  // start at 1, skip header
        {
        QStringList fields = records.at(i).split(QRegExp(",|;"));
        if (fields.length() > 3)
            {
            if (m_Starts.find(fields.at(1).toLong()) != m_Starts.end())
                ++m_IgnoredStarts;
            m_Starts[fields.at(1).toLong()] = fields.at(3).left(8);
            }
        }
}

QString MainWindow::ExtractTime(QString a_Time)
{
    QRegExp re("([0-9]{1,2}:[0-9]{2,2}:[0-9]{2,2})");
    re.indexIn(a_Time);
    QStringList list = re.capturedTexts();
    if (list.count() > 0)
        return list.at(0);
    return a_Time;
}

void MainWindow::LoadFinishes(QString& a_Finishes)
{
    QStringList records = a_Finishes.split(QRegExp("\n|\r|\r\n"));
    m_IgnoredFinishes = 0;

    // Rely on the fact that they are stored in chronological order
    for (int i = 0; i < records.count(); i++)
        {
        QStringList fields = records.at(i).split(";");
        if (fields.length() > 3)
            {
            if (m_Finished.find(fields.at(0).toLong()) != m_Finished.end())
                ++m_IgnoredFinishes;
            m_Finished[fields.at(0).toLong()] = ExtractTime(fields.at(3));
            }
        }
}

void MainWindow::EliminateMatches()
{
    int matched(0), lost(0);
    for (std::map<long, QString>::iterator iter = m_Starts.begin(); iter != m_Starts.end(); iter++)
        {
        if (m_Finished.find(iter->first) == m_Finished.end() ||
        !MatchTime(iter->second, m_Finished.find(iter->first)->second))

            {
            ++lost;
            QString warning = QString("No finish for %1, started %2").arg(iter->first).arg(iter->second);
            emit newWarning(warning);
            }
        else
            ++matched;
        }
    QString msg = QString("Ignored %1 SI cards from start which were re-used later").arg(m_IgnoredStarts);
    emit newWarning(msg);
    msg = QString("Ignored %1 SI cards from finish which were re-used later").arg(m_IgnoredFinishes);
    emit newWarning(msg);
    msg = QString("Eliminated %1 cards which were matched").arg(matched);
    emit newWarning(msg);
    msg = QString("Lost %1 people !").arg(lost);
    emit newWarning(msg);
}


void MainWindow::run()
    {
    emit startRun();
 // Load new file
    QFile tfile(ui->StartFileEdit->text());
    tfile.open(QIODevice::ReadOnly);
    QByteArray bytes = tfile.readAll();
    QString StartString(bytes); // string
    tfile.close();

    QFile tfile2(m_Events[ui->FinishedCombo->currentText()]);
    tfile2.open(QIODevice::ReadOnly);
    bytes = tfile2.readAll();
    QString FinishedString(bytes); // string
    tfile2.close();

// Split Starts and eliminate duplicates
    LoadStarts(StartString);
    LoadFinishes(FinishedString);
    EliminateMatches();
    }
